procedure arg (inputs, template)

string	inputs	=""	{prompt="File with substitution strings"}
string	template=""	{prompt="Template script with markers"}
string	marker	="&"	{prompt="Marker string representing inputs"}
int	nlines	=0	{prompt="Run-mode; number of lines to replicate"}

# @(#) task arg                       Author: R.A. Jansen -- Aug 23 2003
# @(#)
# @(#) IRAF user interface for script building program 'arg'. 'Arg' is a
# @(#) program to facilitate writing long scripts where the same command
# @(#) or set of commands need be repeated for a large number of non-tri
# @(#) vial input values (e.g.,  a list of file or  image names,  object
# @(#) names, numerical constants, etc.). Using 'arg', one only needs to
# @(#) write a template for the script once, in which the variable input
# @(#) is represented by a unique marker character or character sequence
# @(#) that is not used in the script to denote anything other than that
# @(#) input. For each line in input file 'inputs', 'arg' will replicate
# @(#) 'nlines' lines from the template script, while replacing each in-
# @(#) stance of  'marker'  by the text string  read from  'inputs'.  If
# @(#) 'nlines'=0 the entire template script is replicated for each line
# @(#) in 'inputs'.  If 'nlines' is other than 0, the template must have
# @(#) 'nlines' times the number of lines 'inputs' has. Output is to the
# @(#) standard output, but may be redirected to a file. When 'nlines'=0
# @(#) commented lines (starting with a '#')  in the template script are
# @(#) skipped and omitted from the output.
# @(#)
# @(#) Feb 23 1998 -- First 'rjtools'/'DinoSoft' version,  based in part
# @(#)                on earlier code by M.Franx
# @(#) Apr 26 1999 -- Port to linux;  cleaned up code for less forgiving
# @(#)                'g77' (compared to native Fortran Sun Solaris f77
# @(#)                compiler). RAJ.
# @(#) Apr 23 2001 -- 'fc' within IRAF no longer required to compile arg
# @(#)                (to read command line arguments);  added -c switch
# @(#)                to skip commented lines in 'template' for -m0. RAJ
# @(#) Nov 27 2001 -- Wrote unix 'man' page and 'install' script. RAJ
# @(#) Aug 23 2003 -- Wrote IRAF wrapper and help file for 'rjtools_2.12'
# @(#)                distribution. RAJ

begin
	string	cfil, tfil, mrk, cbin
	int	nl, ni, nt

#	Initialize variables...
	cfil	= inputs
	tfil	= template
	mrk	= marker
	nl	= nlines

#	(Re)initialize task parameters...
	inputs	= cfil
	template= tfil
	marker	= mrk
	nlines	= nl

#	Test existence of substitutions file...
	if ( !access(cfil) ) {
	   print ("ERROR: substitutions file \""//cfil//"\" not found!")
	   return
	}

#	Test existence of template script...
	if ( !access(tfil) ) {
	   print ("ERROR: template script \""//tfil//"\" not found!")
	   return
	}

#	Count number of lines in both files...
	count (cfil) | scan (ni)
	count (tfil) | scan (nt)

#	Test consistency of number of lines in template and inputs...
	if ( nl != 0 && nt < (nl*ni) ) {
	   error(1,"template has fewer than 'nlines' x number of lines in 'inputs'!")
	}
	if ( nl != 0 && nt > (nl*ni) ) {
	   error(1,"template has more than 'nlines' x number of lines in 'inputs'!")
	}

#	See if 'arg' executable is present...
	show ("rjbin") | pathnames ("-") | scan (cbin)
	cparse (cbin, delim="!", nsel=2, ligno-, migno-, verb+) | scan (cbin)
	if ( !access(cbin//"arg") ) {
	   error (1, "executable 'arg' not found in \""//cbin//"\"!")
	}

#	Call 'arg' with proper command line arguments...
	if ( access("_tmparg.cl") ) delete ("_tmparg.cl", yes, verify-)
	if ( nl == 0 ) {
	   printf("!"//cbin//"arg -m%d -c "//cfil//" "//tfil//" "//mrk//"\n", nl, > "_tmparg.cl")
	} else {
	   printf("!"//cbin//"arg -m%d "//cfil//" "//tfil//" "//mrk//"\n", nl, > "_tmparg.cl")
	}
	cl < _tmparg.cl

#	Clean-up ...
	delete ("_tmparg.cl", yes, verify-)

end
