procedure markall (image)

string	image	=""		{prompt="Image name"}
real	z1	=INDEF		{prompt="Lower cut level"}
real	z2	=INDEF		{prompt="Upper cut level"}
bool	redispl	=yes		{prompt="(Re)display image first?"}
bool	gal	=yes		{prompt="Mark galaxies?"}
bool	star	=yes		{prompt="Mark stars?"}
bool	cosmic	=yes		{prompt="Mark cosmics?"}
bool	del	=yes		{prompt="Mark CCD defects?"}
bool	append	=no		{prompt="Append to existing position files?"}
bool	verbose	=yes		{prompt="Verbose messages?"}

# @(#) task markall		      Author: R.A. Jansen -- Apr 19 1996
# @(#)
# @(#) task to interactively mark galaxies, stars, cosmic ray events and
# @(#) CCD defects in a displayed image.  This task replaces  'surfphot.
# @(#) mark'.
# @(#)
# @(#) Calls 'rjtools' tasks: 'chkimg','sysinfo','gdate', and 'xdisplay'
# @(#)
# @(#) Jun 21 2000 -- Added support for multi-extension images and image
# @(#)                sections. RAJ.
# @(#) Aug 17 2003 -- Cleaned-up code; wrote help file. RAJ.
# @(#) Apr 22 2004 -- Added usage hints when gal=no and verbose=no. RAJ.

begin
	string	cimg, cimr, gfil, sfil, cfil, dfil
	string	cmd, c, key, arg, val, tst, previous
	file	tmptvmark
	real	zlo, zhi, rrad, x, y, x0, y0, r, t
	int	nc, nr, l

#	Initialize variables...
	cimg  = image
	zlo   = z1
	zhi   = z2
	rrad  = 0.5

#	Check whether tv package is loaded...
	if ( !defpac("images") ) { images }
	if ( !defpac("tv") ) { tv }

#	Check whether input image exists
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimr = chkimg.impath//chkimg.imroot
	cimg = cimr//chkimg.imtype//chkimg.imrpstr

#	Construct the positions file names...
	gfil = cimr//".pos.gal"
	sfil = cimr//".pos.star"
	cfil = cimr//".pos.cosmic"
	dfil = cimr//".pos.del"

#	Determine the image size...
	hselect (cimg, "i_naxis1", "yes") | scan (nc)
	hselect (cimg, "i_naxis2", "yes") | scan (nr)

#	Check whether anything is supposed to be marked...
	if ( !gal && !star && !cosmic && !del ) {
	   error (1,"All mark switches are set to \"no\")!")
	}

#	Opening message...
	if (verbose) {
	   sysinfo() ; gdate(utc=no)
	   printf("MARKALL:  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n")
	   printf("   image = "//cimg//" [%d x %d]\n", nc, nr)
	}

#	If older versions of positions/regions files are present and
#	'append=no', rename them.  The 'verbose' option in task 'copy'
#	seems not to be implemented!
	if ( gal && access(gfil) && !append ) {
	   if ( access(gfil//"~") ) delete (gfil//"~", yes, verify-)
	   print ("Saving existing file \""//gfil//"\" --> \""//gfil//"~\"...")
	   rename (gfil, gfil//"~", field="all")
	}
	if ( star && access(sfil) && !append ) {
	   if ( access(sfil//"~") ) delete (sfil//"~", yes, verify-)
	   print ("Saving existing file \""//sfil//"\" --> \""//sfil//"~\"...")
	   rename (sfil, sfil//"~", field="all")
	}
	if ( cosmic && access(cfil) && !append ) {
	   if ( access(cfil//"~") ) delete (cfil//"~", yes, verify-)
	   print ("Saving existing file \""//cfil//"\" --> \""//cfil//"~\"...")
	   rename (cfil, cfil//"~", field="all")
	}
	if ( del && access(dfil) && !append ) {
	   if ( access(dfil//"~") ) delete (dfil//"~", yes, verify-)
	   print ("Saving existing file \""//dfil//"\" --> \""//dfil//"~\"...")
	   rename (dfil, dfil//"~", field="all")
	}

#	(Re)display image is 'redispl = yes'...
	if (redispl) {
	   xdisplay (cimg, buff=1, z1=zlo, z2=zhi, lsig=3., hsig=15.,
		niter=1, zmode="linear", verbose=no)
	}

#	Mark galaxies in the displayed image...
	if (gal) {
	   if (append && access(gfil) ) {
	      print ("Appending positions to file \""//gfil//"\"...")
	   }
	   if (verbose) {
	      print ("Wait for the cursor cross to appear in the image display area, then")
	      print ("mark all galaxies in the image. The following key-strokes are recog-")
	      print ("nized:  hit v twice      -- circle (center+circumference)")
	      print ("        hit b twice      -- rectangle (diagonally oposite corners)")
	      print ("        hit :rad radlen  -- set radius of circle to radlen")
	      print ("        hit c            -- circle with radius radlen (center)")
	      print ("        hit q            -- quit")
	      print ("NB: after hitting a colon, don't type \"rad\" to quickly.")
	   } else {
	      print ("Mark all galaxies in the image.")
	      print ("\"v\" twice = circle, \"b\" twice = rectangle, \"c\" = circle")
	      print ("of radius radlen, \":rad\" redefine radius, \"q\" = quit")
	   }
	   sleep(1)

	   tmptvmark=mktemp("tvmark")

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		outimage="", deletions="", commands="", mark="plus",
		radii=str(rrad), lengths="0", font="raster", color=201,
		label-, number-, nxoffset=0, nyoffset=0, pointsize=3,
		txsize=1, tolerance=1.5, inter+, > "dev$null", >& "STDERR")

	   list = tmptvmark 
	   previous = ""
	   while( fscan(list,x,y,key,cmd,arg,val) != EOF ) {
	      if ( x < 1. ) x = 1.
	      if ( x > nc ) x = nc
	      if ( y < 1. ) y = 1.
	      if ( y > nr ) y = nr
	      if ( cmd == ":") {
		 l = strlen(cmd)
		 c = substr(cmd,4,l)
		 if ( substr(arg,1,3) == "rad" ) {
		    tst = fscan(val,rrad)
		 } else {
		    print("ERROR: Invalid command \""//arg//"\"")
		    previous = ""
		 }
	      } else if ( cmd == "c" ) {
		 print (x,y,rrad, >> gfil)
	      } else if ( previous == "" ) {
		 previous = cmd
		 x0 = x
		 y0 = y
	      } else if ( previous == cmd ) {
		 if ( previous == "v" ) {
		    r = sqrt((x0-x)*(x0-x) + (y0-y)*(y0-y))
		    print (x0,y0,r, >> gfil)
		    previous = ""
		 } else if ( previous == "b" ) {
		    if ( x0 > x ) {
		       t  = x0
		       x0 = x
		       x  = t
		    }
		    if ( y0 > y ) {   
		       t  = y0
		       y0 = y 
		       y  = t
		    }
		    print (x0,x,y0,y, >> gfil)
		    previous = ""  
		 } else {
		    print ("ERROR: Invalid command \""//cmd//"\"!")
		 }
	      } else { 
		 print ("ERROR: Invalid command sequence \""//previous//"\",\""//cmd//"\"!")
		 previous = ""
	      }
	   }
	   delete (tmptvmark, yes, verify-)
	}

#	Mark all stars (and/or regions affected by star glow) in the image...
	if (star) {
	   if (verbose && !gal ) {
	      print ("Wait for the cursor cross to appear in the image display area, then")
	      print ("mark all stars in the image. The following key-strokes are recog-")
	      print ("nized:  hit v twice      -- circle (center+circumference")
	      print ("        hit b twice      -- rectangle (diagonally oposite corners)")
	      print ("        hit :rad radlen  -- set radius of circle to radlen")
	      print ("        hit c            -- circle with radius radlen (center)")
	      print ("        hit q            -- quit")
	      print ("NB: after hitting a colon, don't type \"rad\" to quickly.")
	   } else {
	      if (!gal) {
		 print ("Mark all stars in the image.")
		 print ("\"v\" twice = circle, \"b\" twice = rectangle, \"c\" = circle")
		 print ("of radius radlen, \":rad\" redefine radius, \"q\" = quit")
	      } else {
		 print ("Wait for the cursor cross to appear in the image display area, then")
		 print ("mark all stars in the image.")
	      }
	   }
	   sleep(1)

	   tmptvmark = mktemp("tvmark")

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		outimage="", deletions="", commands="", mark="point",
		radii=str(rrad), lengths="0", font="raster", color=204,
		label-, number-, nxoffset=0, nyoffset=0, pointsize=3,
		txsize=1, tolerance=1.5, inter+, > "dev$null", >& "STDERR")

	   list = tmptvmark 
	   previous = ""
	   while( fscan(list,x,y,key,cmd,arg,val) != EOF ) {
	      if ( x < 1. ) x = 1.
	      if ( x > nc ) x = nc
	      if ( y < 1. ) y = 1.
	      if ( y > nr ) y = nr
	      if ( cmd == ":") {
		 l = strlen(cmd)
		 c = substr(cmd,4,l)
		 if ( substr(arg,1,3) == "rad" ) {
		    tst = fscan(val,rrad)
		 } else {
		    print("ERROR: Invalid command \""//arg//"\"")
		    previous = ""
		 }
	      } else if ( cmd == "c" ) {
		 print (x,y,rrad, >> sfil)
	      } else if ( previous == "" ) {
		 previous = cmd
		 x0 = x
		 y0 = y
	      } else if ( previous == cmd ) {
		 if ( previous == "v" ) {
		    r = sqrt((x0-x)*(x0-x) + (y0-y)*(y0-y))
		    print (x0,y0,r, >> sfil)
		    previous = ""
		 } else if ( previous == "b" ) {
		    if ( x0 > x ) {
		       t  = x0
		       x0 = x
		       x  = t
		    }
		    if ( y0 > y ) {   
		       t  = y0
		       y0 = y 
		       y  = t
		    }
		    print (x0,x,y0,y, >> sfil)
		    previous = ""  
		 } else {
		    print ("ERROR: Invalid command \""//cmd//"\"!")
		 }
	      } else { 
		 print ("ERROR: Invalid command sequence \""//previous//"\",\""//cmd//"\"!")
		 previous = ""
	      }
	   }
	   delete (tmptvmark, yes, verify-)
	}

#	Mark all pixel affected by cosmic ray hits in the image...
	if (cosmic) {
	   if (verbose && !gal && !star ) {
	      print ("Wait for the cursor cross to appear in the image display area, then")
	      print ("mark all pixels affected by cosmic ray hits in the image.")
	      print ("The following key-strokes are recognized:")
	      print ("        hit v twice      -- circle (center+circumference")
	      print ("        hit b twice      -- rectangle (diagonally oposite corners)")
	      print ("        hit :rad radlen  -- set radius of circle to radlen")
	      print ("        hit c            -- circle with radius radlen (center)")
	      print ("        hit q            -- quit")
	      print ("NB: after hitting a colon, don't type \"rad\" to quickly.")
	   } else {
	      if ( !gal && !star ) {
		 print ("Mark all cosmics in the image.")
		 print ("\"v\" twice = circle, \"b\" twice = rectangle, \"c\" = circle")
		 print ("of radius radlen, \":rad\" redefine radius, \"q\" = quit")
	      } else {
		 print ("Wait for the cursor cross to appear in the image display area, then")
		 print ("mark all pixels affected by cosmic ray hits in the image.")
	      }
	   }
	   sleep(1)

	   tmptvmark = mktemp("tvmark")

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		outimage="", deletions="", commands="", mark="point",
		radii="1.", lengths="0", font="raster", color=206,
		label-, number-, nxoffset=0, nyoffset=0, pointsize=1,
		txsize=1, tolerance=1.5, inter+, > "dev$null", >& "STDERR")

	   list = tmptvmark 
	   previous = ""
	   while( fscan(list,x,y,key,cmd,arg,val) != EOF ) {
	      if ( x < 1. ) x = 1.
	      if ( x > nc ) x = nc
	      if ( y < 1. ) y = 1.
	      if ( y > nr ) y = nr
	      if ( cmd == ":") {
		 l = strlen(cmd)
		 c = substr(cmd,4,l)
		 if ( substr(arg,1,3) == "rad" ) {
		    tst = fscan(val,rrad)
		 } else {
		    print("ERROR: Invalid command \""//arg//"\"")
		    previous = ""
		 }
	      } else if ( cmd == "c" ) {
		 print (x,y,rrad, >> cfil)
	      } else if ( previous == "" ) {
		 previous = cmd
		 x0 = x
		 y0 = y
	      } else if ( previous == cmd ) {
		 if ( previous == "v" ) {
		    r = sqrt((x0-x)*(x0-x) + (y0-y)*(y0-y))
		    print (x0,y0,r, >> cfil)
		    previous = ""
		 } else if ( previous == "b" ) {
		    if ( x0 > x ) {
		       t  = x0
		       x0 = x
		       x  = t
		    }
		    if ( y0 > y ) {   
		       t  = y0
		       y0 = y 
		       y  = t
		    }
		    print (x0,x,y0,y, >> cfil)
		    previous = ""  
		 } else {
		    print ("ERROR: Invalid command \""//cmd//"\"!")
		 }
	      } else { 
		 print ("ERROR: Invalid command sequence \""//previous//"\",\""//cmd//"\"!")
		 previous = ""
	      }
	   }
	   delete (tmptvmark, yes, verify-)
	}


#	Mark all CCD defects and other deletions regions in the image...
	if (del) {
	   if (verbose && !gal && !star && !cosmic ) {
	      print ("Wait for the cursor cross to appear in the image display area, then")
	      print ("mark all defects in the image. The following key-strokes are recog-")
	      print ("nized:  hit v twice      -- circle (center+circumference")
	      print ("        hit b twice      -- rectangle (diagonally oposite corners)")
	      print ("        hit :rad radlen  -- set radius of circle to radlen")
	      print ("        hit c            -- circle with radius radlen (center)")
	      print ("        hit q            -- quit")
	      print ("NB: after hitting a colon, don't type \"rad\" to quickly.")
	   } else {
	      print ("Wait for the cursor cross to appear in the image display area, then")
	      print ("mark all CCD defects and other blemishes in the image.")
	   }
	   sleep(1)

	   tmptvmark = mktemp("tvmark")

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		outimage="", deletions="", commands="", mark="point",
		radii=str(rrad), lengths="0", font="raster", color=207,
		label-, number-, nxoffset=0, nyoffset=0, pointsize=3,
		txsize=1, tolerance=1.5, inter+, > "dev$null", >& "STDERR")

	   list = tmptvmark 
	   previous = ""
	   while( fscan(list,x,y,key,cmd,arg,val) != EOF ) {
	      if ( x < 1. ) x = 1.
	      if ( x > nc ) x = nc
	      if ( y < 1. ) y = 1.
	      if ( y > nr ) y = nr
	      if ( cmd == ":") {
		 l = strlen(cmd)
		 c = substr(cmd,4,l)
		 if ( substr(arg,1,3) == "rad" ) {
		    tst = fscan(val,rrad)
		 } else {
		    print("ERROR: Invalid command \""//arg//"\"")
		    previous = ""
		 }
	      } else if ( cmd == "c" ) {
		 print (x,y,rrad, >> dfil)
	      } else if ( previous == "" ) {
		 previous = cmd
		 x0 = x
		 y0 = y
	      } else if ( previous == cmd ) {
		 if ( previous == "v" ) {
		    r = sqrt((x0-x)*(x0-x) + (y0-y)*(y0-y))
		    print (x0,y0,r, >> dfil)
		    previous = ""
		 } else if ( previous == "b" ) {
		    if ( x0 > x ) {
		       t  = x0
		       x0 = x
		       x  = t
		    }
		    if ( y0 > y ) {   
		       t  = y0
		       y0 = y 
		       y  = t
		    }
		    print (x0,x,y0,y, >> dfil)
		    previous = ""  
		 } else {
		    print ("ERROR: Invalid command \""//cmd//"\"!")
		 }
	      } else { 
		 print ("ERROR: Invalid command sequence \""//previous//"\",\""//cmd//"\"!")
		 previous = ""
	      }
	   }
	   delete (tmptvmark, yes, verify-)
	}

	list = ""
	print ("\nMARKALL: Finished.")

end
